#include <STC32G.H>
#include "zf_spi.h"
#include "zf_gpio.h"
#include "OLED.h"
#include "OLED_Font.h"
#include "delay.h"

/*ųʼ*/
static void OLED_GPIO_Init(void)
{
	gpio_mode(LCD_RES_GPIO_PIN,GPO_PP);
	gpio_mode(LCD_DC_GPIO_PIN,GPO_PP);
	gpio_mode(LCD_CS_GPIO_PIN,GPO_PP);
	
	LCD_RES_Set;
	LCD_DC_Set;
	LCD_CS_Set;
}

/**
  * @brief  SPIһֽ
  * @param  Byte Ҫ͵һֽ
  * @retval 
  */
void OLED_SPI_SendByte(uint8_t Byte)
{
	LCD_CS_Clr;
	spi_mosi(Byte);
	LCD_CS_Set;
}

/**
  * @brief  OLEDд
  * @param  Command Ҫд
  * @retval 
  */
void OLED_WriteCommand(uint8_t Command)
{
	LCD_CS_Clr;
	LCD_DC_Clr;
	OLED_SPI_SendByte(Command);
	LCD_CS_Set;
}

/**
  * @brief  OLEDд
  * @param  Data Ҫд
  * @retval 
  */
void OLED_WriteData(uint8_t Data)
{
	LCD_CS_Clr;
	LCD_DC_Set;
	OLED_SPI_SendByte(Data);
	LCD_CS_Set;
}

/**
  * @brief  OLEDùλ
  * @param  Y ϽΪԭ㣬·꣬Χ0~7
  * @param  X ϽΪԭ㣬ҷ꣬Χ0~127
  * @retval 
  */
void OLED_SetCursor(uint8_t Y, uint8_t X)
{
	OLED_WriteCommand(0xB0 | Y);					//Yλ
	OLED_WriteCommand(0x10 | ((X & 0xF0) >> 4));	//Xλø4λ
	OLED_WriteCommand(0x00 | (X & 0x0F));			//Xλõ4λ
}

/**
  * @brief  OLED
  * @param  
  * @retval 
  */
void OLED_Clear(void)
{  
	uint8_t i, j;
	for (j = 0; j < 8; j++)
	{
		OLED_SetCursor(j, 0);
		for(i = 0; i < 128; i++)
		{
			OLED_WriteData(0x00);
		}
	}
}

/**
  * @brief  OLEDʾһַ
  * @param  Line λãΧ1~4
  * @param  Column λãΧ1~16
  * @param  Char ҪʾһַΧASCIIɼַ
  * @retval 
  */
void OLED_ShowChar(uint8_t Line, uint8_t Column, char Char)
{      	
	uint8_t i;
	OLED_SetCursor((Line - 1) * 2, (Column - 1) * 8);		//ùλϰ벿
	for (i = 0; i < 8; i++)
	{
		OLED_WriteData(OLED_F8x16[Char - ' '][i]);			//ʾϰ벿
	}
	OLED_SetCursor((Line - 1) * 2 + 1, (Column - 1) * 8);	//ùλ°벿
	for (i = 0; i < 8; i++)
	{
		OLED_WriteData(OLED_F8x16[Char - ' '][i + 8]);		//ʾ°벿
	}
}

/**
  * @brief  OLEDʾַ
  * @param  Line ʼλãΧ1~4
  * @param  Column ʼλãΧ1~16
  * @param  String ҪʾַΧASCIIɼַ
  * @retval 
  */
void OLED_ShowString(uint8_t Line, uint8_t Column, char *String)
{
	uint8_t i;
	for (i = 0; String[i] != '\0'; i++)
	{
		OLED_ShowChar(Line, Column + i, String[i]);
	}
}

/**
  * @brief  OLEDη
  * @retval ֵXYη
  */
uint32_t OLED_Pow(uint32_t X, uint32_t Y)
{
	uint32_t Result = 1;
	while (Y--)
	{
		Result *= X;
	}
	return Result;
}

/**
  * @brief  OLEDʾ֣ʮƣ
  * @param  Line ʼλãΧ1~4
  * @param  Column ʼλãΧ1~16
  * @param  Number Ҫʾ֣Χ0~4294967295
  * @param  Length ҪʾֵĳȣΧ1~10
  * @retval 
  */
void OLED_ShowNum(uint8_t Line, uint8_t Column, uint32_t Number, uint8_t Length)
{
	uint8_t i;
	for (i = 0; i < Length; i++)							
	{
		OLED_ShowChar(Line, Column + i, Number / OLED_Pow(10, Length - i - 1) % 10 + '0');
	}
}

/**
  * @brief  OLEDʾ֣ʮƣ
  * @param  Line ʼλãΧ1~4
  * @param  Column ʼλãΧ1~16
  * @param  Number Ҫʾ֣Χ-2147483648~2147483647
  * @param  Length ҪʾֵĳȣΧ1~10
  * @retval 
  */
void OLED_ShowSignedNum(uint8_t Line, uint8_t Column, int32_t Number, uint8_t Length)
{
	uint8_t i;
	uint32_t Number1;
	if (Number >= 0)
	{
		OLED_ShowChar(Line, Column, '+');
		Number1 = Number;
	}
	else
	{
		OLED_ShowChar(Line, Column, '-');
		Number1 = -Number;
	}
	for (i = 0; i < Length; i++)							
	{
		OLED_ShowChar(Line, Column + i + 1, Number1 / OLED_Pow(10, Length - i - 1) % 10 + '0');
	}
}

/**
  * @brief  OLEDʾ֣ʮƣ
  * @param  Line ʼλãΧ1~4
  * @param  Column ʼλãΧ1~16
  * @param  Number Ҫʾ֣Χ0~0xFFFFFFFF
  * @param  Length ҪʾֵĳȣΧ1~8
  * @retval 
  */
void OLED_ShowHexNum(uint8_t Line, uint8_t Column, uint32_t Number, uint8_t Length)
{
	uint8_t i, SingleNumber;
	for (i = 0; i < Length; i++)							
	{
		SingleNumber = Number / OLED_Pow(16, Length - i - 1) % 16;
		if (SingleNumber < 10)
		{
			OLED_ShowChar(Line, Column + i, SingleNumber + '0');
		}
		else
		{
			OLED_ShowChar(Line, Column + i, SingleNumber - 10 + 'A');
		}
	}
}

/**
  * @brief  OLEDʾ֣ƣ
  * @param  Line ʼλãΧ1~4
  * @param  Column ʼλãΧ1~16
  * @param  Number Ҫʾ֣Χ0~1111 1111 1111 1111
  * @param  Length ҪʾֵĳȣΧ1~16
  * @retval 
  */
void OLED_ShowBinNum(uint8_t Line, uint8_t Column, uint32_t Number, uint8_t Length)
{
	uint8_t i;
	for (i = 0; i < Length; i++)							
	{
		OLED_ShowChar(Line, Column + i, Number / OLED_Pow(2, Length - i - 1) % 2 + '0');
	}
}

/**
  * @brief  OLEDʼ
  * @param  
  * @retval 
  */
void OLED_Init(void)
{
	delay_ms(100);
	OLED_GPIO_Init();			//˿ڳʼ
	
	OLED_WriteCommand(0xAE);	//رʾ
	
	OLED_WriteCommand(0xD5);	//ʾʱӷƵ/Ƶ
	OLED_WriteCommand(0x80);
	
	OLED_WriteCommand(0xA8);	//ö·
	OLED_WriteCommand(0x3F);
	
	OLED_WriteCommand(0xD3);	//ʾƫ
	OLED_WriteCommand(0x00);
	
	OLED_WriteCommand(0x40);	//ʾʼ
	
	OLED_WriteCommand(0xA1);	//ҷ0xA1 0xA0ҷ
	
	OLED_WriteCommand(0xC8);	//·0xC8 0xC0·

	OLED_WriteCommand(0xDA);	//COMӲ
	OLED_WriteCommand(0x12);
	
	OLED_WriteCommand(0x81);	//öԱȶȿ
	OLED_WriteCommand(0xCF);

	OLED_WriteCommand(0xD9);	//Ԥ
	OLED_WriteCommand(0xF1);

	OLED_WriteCommand(0xDB);	//VCOMHȡѡ񼶱
	OLED_WriteCommand(0x30);

	OLED_WriteCommand(0xA4);	//ʾ/ر

	OLED_WriteCommand(0xA6);	///תʾ

	OLED_WriteCommand(0x8D);	//ó
	OLED_WriteCommand(0x14);

	OLED_WriteCommand(0xAF);	//ʾ
		
	OLED_Clear();				//OLED
}
